@@ -1,7 +1,7 @@
 /*
  * arping
  *
- * By marvin@rootbusters.net
+ * By Thomas Habets <thomas@habets.pp.se>
  *
  * ARP 'ping' utility
  *
@@ -12,10 +12,10 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c,v 1.44 2001/09/09 22:47:42 marvin Exp $
+ * $Id: arping.c,v 1.50 2002/01/20 23:14:08 marvin Exp $
  */
 /*
- *  Copyright (C) 2000 Marvin (marvin@rootbusters.net)
+ *  Copyright (C) 2000-2002 Thomas Habets <thomas@habets.pp.se>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -31,9 +31,12 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <sys/time.h>
+
 #ifndef ETH_ALEN
 #define ETH_ALEN 6
 #endif
@@ -47,6 +50,11 @@
 #include "freebsd.h" 
 #endif
 
+#if MACOSX
+#include <sys/socket.h>
+#include "freebsd.h" 
+#endif
+
 #if USE_NETIF
 #include <net/if.h>
 #include <net/if_arp.h>
@@ -70,7 +78,7 @@
 #define DEBUG(a)
 #endif
 
-const float version = 1.02;
+const float version = 1.03;
 
 struct ether_addr *mymac;
 static u_char eth_xmas[ETH_ALEN] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
@@ -78,6 +86,7 @@ static u_char eth_null[ETH_ALEN] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
 static u_char eth_target[ETH_ALEN];
 static u_char eth_source[ETH_ALEN];  // only used in main() but it belongs here
 
+static struct timeval lastpacketsent;
 static const u_int ip_xmas = 0xffffffff;
 
 static pcap_t *pcap;
@@ -93,6 +102,7 @@ static u_int32_t dip = 0;
 static u_char *packet;
 struct libnet_link_int *linkint;
 
+static unsigned int beep = 0;
 static unsigned int verbose = 0;
 static unsigned int numsent = 0;
 static unsigned int numrecvd = 0;
@@ -130,11 +140,12 @@ static void sigint(int i)
 
 static void usage(int ret)
 {
-	printf("arping %1.2f [ -qvrRd0bp ] [ -S <host/ip> ] [ -T <host/ip ]"
-	       " [ -s <MAC> ]\n"
-	       "            [ -t <MAC> ] [ -c <count> ] [ -i <interface> ] "
-	       "<host/ip/MAC | -B>\n",
+	printf("ARPing %1.2f, by Thomas Habets <thomas@habets.pp.se>\n",
 	       version);
+	printf("usage: arping [ -qvrRd0bp ] [ -S <host/ip> ] [ -T <host/ip ]"
+	       " [ -s <MAC> ]\n"
+	       "              [ -t <MAC> ] [ -c <count> ] [ -i <interface> ] "
+	       "<host/ip/MAC | -B>\n");
 	exit(ret);
 }
 
@@ -181,12 +192,37 @@ static void alasend(int i)
 		fprintf(stderr, "libnet_write_link_layer(): error\n");
 		exit(1);
 	}
+	if (gettimeofday(&lastpacketsent, NULL)) {
+		fprintf(stderr, "arping: %s\n", strerror(errno));
+		exit(1);
+	}
 	alarm(1);
 #if SOLARIS
 	signal(SIGALRM, alasend);
 #endif
 }
 
+/*
+ * NOTE: not re-entrant
+ */
+static char* tvtoda(const struct timeval *tv, const struct timeval *tv2)
+{
+	static char buf[128];
+	double f,f2;
+
+	f = tv->tv_sec + (double)tv->tv_usec / 1000000;
+	f2 = tv2->tv_sec + (double)tv2->tv_usec / 1000000;
+	f = (f2 - f) * 1000000;
+	if (f < 1000) {
+		sprintf(buf, "%.3f usec", f);
+	} else if (f < 1000000) {
+		sprintf(buf, "%.3f msec", f / 1000);
+	} else {
+		sprintf(buf, "%.3f sec", f / 1000000);
+	}
+	return buf;
+}
+
 static void handlepacket(const char *unused, struct pcap_pkthdr *h,
 			 u_char *packet)
 {
@@ -196,9 +232,15 @@ static void handlepacket(const char *unused, struct pcap_pkthdr *h,
 	struct icmphdr *hicmp;
 	unsigned int c;
 	unsigned char *cp;
+	struct timeval recvtime;
 
 	DEBUG(printf("handlepacket()\n"));
 
+	if (gettimeofday(&recvtime, NULL)) {
+		fprintf(stderr, "arping: %s\n", strerror(errno));
+		exit(1);
+	}
+
 	eth = (struct ethhdr*)packet;
 
 	if (searchmac) {
@@ -228,19 +270,28 @@ static void handlepacket(const char *unused, struct pcap_pkthdr *h,
 								  0));
 				}
 				if (!rawoutput) {
-					printf("%s",libnet_host_lookup(hip->saddr,0));
+					/*
+					 * ugly code due to non-aligned saddr (bus error on sparc)
+					 */
+					u_int32_t tmp;
+					memcpy(&tmp, &hip->saddr, sizeof(u_int32_t));
+					printf("%s",libnet_host_lookup(tmp,0));
 					printf(" (");
 					for (c = 0; c < ETH_ALEN-1; c++) {
 						printf("%.2x:", *cp++);
 					}
-					printf("%.2x): icmp_seq=%d", *cp,
-					       hicmp->un.echo.sequence);
+					printf("%.2x): icmp_seq=%d time=%s", *cp,
+					       hicmp->un.echo.sequence,
+					       tvtoda(&lastpacketsent, &recvtime));
+				}
+				if (beep) {
+					printf("\a");
 				}
 				printf("\n");
 			}
 		}
 	} else {
-		// ping ip
+		/* ping ip */
 		harp = (struct arphdr*)((char*)eth + sizeof(struct libnet_ethernet_hdr));
 		if ((htons(harp->ar_op) == ARPOP_REPLY)
 		    && (htons(harp->ar_pro) == ETH_P_IP)
@@ -273,16 +324,17 @@ static void handlepacket(const char *unused, struct pcap_pkthdr *h,
 						}
 						printf("%.2x", *cp);
 					}
-				}
-				if (!rawoutput) {
-					printf(" (%s): index=%d",
-					       libnet_host_lookup(ip, 0),
-					       numrecvd);
-				}
-				numrecvd++;
-				if (!quiet) {
+					if (!rawoutput) {
+						printf(" (%s): index=%d time=%s",
+						       libnet_host_lookup(ip, 0),
+						       numrecvd, tvtoda(&lastpacketsent, &recvtime));
+					}
+					if (beep) {
+						printf("\a");
+					}
 					printf("\n");
 				}
+				numrecvd++;
 			}
 		}
 	}
@@ -308,12 +360,16 @@ int main(int argc, char **argv)
 	
 	DEBUG(printf("main()\n"));
 
-	strcpy(ebuf, "no error");
+	strncpy(ebuf, "no error", LIBNET_ERRBUF_SIZE);
+	ebuf[LIBNET_ERRBUF_SIZE -1 ] = 0;
 
 	memcpy(eth_target, eth_xmas, ETH_ALEN);
 
-	while ((c = getopt(argc, argv, "0bdS:T:Bvhi:rRc:qs:t:p")) != EOF) {
+	while ((c = getopt(argc, argv, "0bdS:T:Bvhi:rRc:qs:t:pa")) != EOF) {
 		switch (c) {
+		case 'a':
+			beep = 1;
+			break;
 		case 'v':
 			verbose++;
 			break;
